#!/usr/bin/env python

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os

def write_file (path, contents):
    """Write a file to @path containing @contents.

    Care is taken to ensure that the existing file is only
    overwritten if the new file can be successfully written.
    """
    temp_path = path + ".new"
    try:
        f = file (temp_path, "w")
    except:
        temp_path = None
        f = file (path, "w")

    try:
        f.write (contents)
        f.close ()
    except:
        if not temp_path is None:
            os.remove (temp_path)
        raise

    if not temp_path is None:
        os.rename (temp_path, path)

def read_file (path, must_exist = True):
    """Read the contents of the file specified by @path
    and return the contents as a single string.

    If @must_exist is #False, then an empty string is
    returned if the file does not exist.
    """
    try:
        f = file (path, "r")
    except IOError, e:
        if must_exist or e.errno != errno.ENOENT:
            raise
        return ""
    
    contents = f.read ()
    f.close ()

    return contents
