#!/usr/bin/env python

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import os.path
import exceptions
import struct
import logger
from config import *

class SysError (exceptions.Exception):
    def __init__ (self, args):
        exceptions.Exception.__init__ (self)
        self.args = args

# Taken from anaconda
def get_mem_size ():
    lines = file ("/proc/meminfo").readlines ()
    for line in lines:
        if line.startswith ("MemTotal:"):
            return int (line.split()[1])

    raise SysError (_("Cannot determine size of system memory"))

# Taken from anaconda
def get_swap_size ():
    mem = get_mem_size () / 1024
    mem = ((mem / 16) + 1) * 16

    logger.verbose ("Detected %sM of memory" % mem)

    if mem < 128:
        minswap = 96
        maxswap = 192
    else:
        if mem > 1000:
            minswap = 1000
            maxswap = 2000
        else:
            minswap = mem
            maxswap = 2*mem

    logger.verbose ("Suggesting swap size between %sM and %sM" % (minswap, maxswap))
    
    return maxswap * 1024 * 1024

#
# FIXME: this should really call ped_partition_get_path()
#        but it's not available through pyparted atm
#
def get_partition_path (part):
    return part.geom.dev.path + str (part.num)

#
# FIXME: this is a pathetic hack
#
def get_partition_udev_path (part):
    parent = part.geom.dev.path[1:]
    if parent.find ("/") > 0:
        parent = parent[parent.index ("/")+1:]
    return "/block/" + parent + "/" + parent + str (part.num)

def get_partition_parent_path (part_path):
    devdir = os.path.dirname (part_path)
    part_name = os.path.basename (part_path)

    for blockdev in os.listdir ("/sys/block"):
        for f in os.listdir (os.path.join ("/sys/block", blockdev)):
            if f == part_name:
                return os.path.join (devdir, blockdev)

    raise SysError (_("Cannot find parent device of partition '%s'") % part_path)

#
# This is the same hack anaconda does
#
def get_bios_number (device):
    f = file (device, "r")
    f.seek (MBR_SIG_OFFSET)
    dev_sig = "0x%x" % struct.unpack ("I", f.read (4))
    f.close ()

    logger.verbose ("Looking through EDD info for device with "
                    "mbr_signature == %s" % dev_sig)
    
    if not os.path.exists (EDD_DIR):
        logger.run_command (MODPROBE_PATH + " edd")

    for d in os.listdir (EDD_DIR):
        if not d.startswith (EDD_PREFIX):
            continue

        bios_num = int (d[len(EDD_PREFIX):])
        
        mbr_sig = file (os.path.join (EDD_DIR, d, "mbr_signature")).read().strip()

        logger.verbose ("Drive num %d has signature %s" % (bios_num, mbr_sig))

        if mbr_sig == dev_sig:
            return bios_num - 80

    raise SysError (_("Cannot determine BIOS number for '%s'") % device)
