#!/usr/bin/env python

#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import os
import os.path
import locale
import gettext
import optparse
import updater
from config import *

from stacaccli.common import fileutils
from stacaccli.common import logger
from stacaccli.common import shvar
from stacaccli.common.config import *

def main (args):
    locale.setlocale (locale.LC_ALL, "")
    gettext.install (PACKAGE, os.path.join (DATADIR, "locale"))

    parser = optparse.OptionParser (version = "%prog " + VERSION,
                                    usage = _("%prog\n"));

    group = optparse.OptionGroup (parser, _("Config Options"))
    group.add_option ("--repository",
                      dest = "repo",
                      help = _("Update the image from the supplied URL"),
                      metavar = _("URL"))
    group.add_option ("--image",
                      dest = "image",
                      help = _("Update the image with the supplied name"),
                      metavar = _("NAME"))
    parser.add_option_group (group)

    group = optparse.OptionGroup (parser, _("Daemon Options"))
    group.add_option ("--nofork",
                      action = "store_true",
                      dest = "nofork",
                      default = False,
                      help = _("Do not fork and become a daemon process"))
    group.add_option ("--debug",
                      action = "store_true",
                      dest = "debug",
                      default = False,
                      help = _("Print debugging messages to stdout.  Implies --nofork"))
    group.add_option ("--pidfile",
                      dest = "pidfile",
                      help = _("Store the daemon's process ID in the named file"),
                      metavar = _("PIDFILE"))
    parser.add_option_group (group)
    
    (options, args) = parser.parse_args (args)

    logger.set_verbose_output (options.debug)
    
    if options.debug:
        options.nofork = True

    if not options.nofork:
        pid = os.fork ()
        if pid != 0: # parent
            os._exit (0)

            os.setsid ()

            os.chdir ("/")
            os.umask (0022)

            if not options.debug:
                fd = os.open ("/dev/null", os.O_RDWR)

                os.dup2 (fd, 0)
                os.dup2 (fd, 1)
                os.dup2 (fd, 2)

                os.close (fd)

    if options.pidfile:
        fileutils.write_file (options.pidfile, str (os.getpid ()))

    conf = shvar.Config ()

    if not options.repo:
        if not conf.repository_url:
            raise updater.UpdaterError (_("No URL for image repository supplied"))
        options.repo = conf.repository_url

    if not options.image:
        if not conf.image:
            raise updater.UpdaterError (_("No image name supplied"))
        options.image = conf.image_name

    upd = updater.Update (options.repo, options.image)
    
    try:
        import time

        #
        # FIXME: either make this a configurable polling
        #        interval or make it something which is
        #        run as a cronjob instead of a daemon
        #
        
        while True:
            upd.run ()
            time.sleep (10)
    finally:
        upd.cleanup ()
        if options.pidfile and os.path.exists (options.pidfile):
            os.remove (options.pidfile)
